    %MACRO calc_final_stats_cox_central;

  %PUT;
   %PUT -----   Start of the macro calc_final_stats_cox_central  iter_nb=&iter_nb. ;
   %PUT;

		%LOCAL  _cl  modelfit_stats ;

   %IF &iter_nb. = %eval(&iter_nb_at_conv. + 1) %THEN
                                            %DO;

												/* If stratification vars do not have var dp_cd then create dataset with baseline hazard function
            &prefix0.baseln_hazard  in  infolder so that it can be send to DPs. Otherwise 
		    created only final dataset in msoc library.
		*/

%IF &pos_dp_cd_in_strata_vars. =0 
  %THEN	%LET tmp_info_baseln_hazard= infolder.&prefix0.baseln_hazard(KEEP=&strata_vars. &dependent_vars. _Cum_Hazard0_	);
%ELSE     %LET tmp_info_baseln_hazard=;


	 %IF %LENGTH(&strata_vars.) >0 %THEN
	                     %DO;	
   DATA &tmp_info_baseln_hazard. 
              msoc.&prefix0.baseln_hazard ;					
     LENGTH Stratum 8;
       MERGE  dplocal.&prefix0.baseln_hazard
	  			      dplocal.&prefix0.Stratum_map;
	 	 BY &strata_vars.;
	 RUN;
						%END;
	 %ELSE
	                   %DO;
  DATA &tmp_info_baseln_hazard. 
             msoc.&prefix0.baseln_hazard(KEEP=Stratum &strata_vars. &dependent_vars. _Cum_Hazard0_	)   ;					
     LENGTH Stratum 8;
       SET dplocal.&prefix0.baseln_hazard;
		 Stratum=1;
	 RUN;
					   %END;
	 
	PROC SORT DATA=msoc.&prefix0.baseln_hazard ;
	 BY  Stratum &dependent_vars.;
	RUN;

	   /* Calculate mean of linear predictor _XBETA_ */
   	PROC SCORE DATA= dplocal.&prefix0.mean_vars_by_st
					     SCORE=msoc.&prefix0.Model_Coeff	  TYPE=PARMS 
						OUT=&prefix0.mean_XBETA_
		;
       VAR &independent_vars.;
    run;

		   /* Calculate survival function at mean value of covariates*/

	DATA  msoc.&prefix0.baseln_survival;
	 MERGE msoc.&prefix0.baseln_hazard(KEEP=Stratum &strata_vars. &dependent_vars.  _Cum_Hazard0_)
	               &prefix0.mean_XBETA_(KEEP=Stratum &strata_vars. _XBETA_)
				;
	  BY Stratum ;

	    _SURVIVAL_at_MEAN=EXP( -_Cum_Hazard0_ *EXP(_XBETA_));

	   RENAME  _XBETA_=_XBETA__MEAN;
	RUN;	   

          DATA  &prefix.Neg2LogLike;
			  LENGTH iter_nb Neg2LogLike 8;
				SET DPLocal.&prefix.resid_sum(KEEP=_LL__Sum _L__Sum);
				iter_nb=&iter_nb.-1;
	             LogLikelihood=_L__Sum ;
				Neg2LogLike=-2*_L__Sum;
		   RUN;

			DATA    dplocal.&prefix0.Iter_Parms_hist;
			 MERGE  dplocal.&prefix0.Iter_Parms_hist
                             &prefix.Neg2LogLike
						;
                BY iter_nb;
 		   RUN;
	
	   DATA msoc.&prefix0.Iter_Parms_hist;
	  	 SET dplocal.&prefix0.Iter_Parms_hist;
		  Iteration=iter_nb;
	  RUN;

	 DATA 	&prefix0.Event_Total;
          SET msoc.&prefix0.cens_sum(KEEP=Event) END=last;
			 IF last;
	 RUN;

	DATA msoc.&prefix0.resid_sum;
	 SET DPLocal.&prefix.resid_sum;
	  IF _N_=1 THEN	
								 DO;
       SET msoc.&prefix0.Model_Coeff(KEEP=_IN_ _P_  );
	   SET &prefix0.Event_Total;
								END;
	  Neg2LogLike= -2* _L__Sum ;
	 _AIC_=-2* _L__Sum+2*_P_;
     _SBC_=-2* _L__Sum+_P_*LOG(Event);

	   	_2LogLike_Ratio=2*(_L__Sum -_L0__Sum);
	  	PValue_Glob_Null_Test=1-PROBCHI( _2LogLike_Ratio, _IN_);
	 RUN;

	  PROC DATASETS LIB=msoc NOLIST;
			 MODIFY &prefix0.resid_sum;
				 LABEL  Neg2LogLike="-2Log L"
							_AIC_ ="AIC (smaller is better)"
			                _SBC_="SBC (smaller is better)"
			 ;
		  RUN;

	 %LET modelfit_stats=%SYSFUNC(COMPBL(Neg2LogLike   _AIC_   _SBC_ ));  ;

	     DATA &prefix0.modelfit_tr;
		  LENGTH &modelfit_stats. 8 ;
			 SET 	msoc.&prefix0.resid_sum;

			 KEEP &modelfit_stats.;
		 RUN;

		 
		 PROC TRANSPOSE DATA=&prefix0.modelfit_tr 
                                        OUT=&prefix0.modelfit1(RENAME=(COL1=Value));

        RUN;

		 DATA msoc.&prefix0.modelfit;
		 LENGTH _NAME_  Criterion $ 32  ;
		  SET &prefix0.modelfit1;
			  IF _N_=1 THEN SET msoc.&prefix0.resid_sum(KEEP=_L0__Sum);

		         Criterion=_LABEL_;	
                 WithoutCovariates=-2*_L0__Sum; 
                 WithCovariates=Value;
			  FORMAT  WithoutCovariates WithCovariates 9.3;
			KEEP _NAME_ Criterion WithoutCovariates WithCovariates ;
		  RUN;    
	 
		  DATA msoc.&prefix0.Glob_Null_ChiSq	;
		   LENGTH Test $ 16 ChiSq DF ProbChiSq  8;
		    SET 	msoc.&prefix0.resid_sum(KEEP=_2LogLike_Ratio _IN_ PValue_Glob_Null_Test 
															     RENAME=(_2LogLike_Ratio= ChiSq _IN_=DF PValue_Glob_Null_Test=ProbChiSq)
	                                                            );
			Test="Likelihood Ratio";
			 FORMAT ProbChiSq  PVALUE6.4;
			    LABEL 	Test="Test"
	                          ChiSq="Chi-Square"	
							  DF="DF"
							  ProbChiSq="Pr > Chi-Square"
				              ;
		   RUN;

		   /* Calculate covariance matrix of estimates as inverse of Hessian*/

  DATA 	msoc.&prefix0.cov_est;
	LENGTH Variable $ 32;
  	 SET DPLocal.&prefix.Ginv_Hessian;
   RUN;

     /* Calculate StErr of parms. estimates from diagonal elements of cov matrix */

		   DATA &prefix0.StdErr_VarN;
			 LENGTH   Var_Numb 4;

		      SET msoc.&prefix0.cov_est;
				ARRAY indep{*} &Intercept. &independent_vars. ;
	
					 DO Var_Numb=1 TO DIM(indep);			
				      IF STRIP(LOWCASE(Variable))=STRIP(LOWCASE( VNAME(indep{Var_Numb}) ))	
					   THEN DO;

						  /*Negative values are not expected. This is just defensive coding.*/

				IF 	indep{Var_Numb}>=0	THEN	StdErr=SQRT(indep{Var_Numb});
				ELSE 											 StdErr=.;					  
     							OUTPUT;
							   END;
				 END; 
				 FORMAT StdErr D11.3;
				KEEP Variable StdErr Var_Numb;
		  RUN;
			  
		   /* Bring together Parm estimates  iter_nb=&iter_nb_at_conv.  with StdErr calculateed at 
		     iter_nb=&iter_nb_at_conv. +1. The extra iteration after convergence is necessary because 
		     at convergence we only have cov matrix  evaluated at \beta_{&iter_nb_at_conv.-1} 
		  */

	DATA  msoc.&prefix0.P_Est ;
	  MERGE &prefix.P_Est_varN_prev
	                  &prefix0.StdErr_VarN
                      ;
		BY 	Var_Numb;

		ChiSq=(Estimate/StdErr)**2;
		ProbChiSq=1-PROBCHI( ChiSq, df);
		HazardRatio=EXP(Estimate);

		alpha=&alpha.;
		Quantile=PROBIT((1- alpha/2));
	    LowerCL=Estimate-Quantile*StdErr;
		UpperCL=Estimate+Quantile*StdErr;
		HR_LowerCL=EXP(LowerCL);
		HR_UpperCL=EXP(UpperCL);
	RUN;

	DATA  _NULL_;
		   CALL SYMPUT("_cl",STRIP(PUT(100*(1.0-&alpha.),best12.)));
	RUN;
			 
			   %PUT _cl=&_cl.;

	PROC DATASETS LIB=msoc NOLIST;
			 MODIFY &prefix0.P_Est;
				 LABEL 
				 Variable="Parameter"
				  Estimate="Parameter Estimate"                                   
                  StdErr="Standard Error"                                         
                  ChiSq="Chi-Square"
                  ProbChiSq="Pr > Chi-Square"                                           
                  LowerCL="Lower &_cl.% CL Parameter"                                
                  UpperCL="Upper &_cl.% CL Parameter" 
                  HazardRatio="Hazard Ratio"
                  HR_LowerCL="Lower &_cl.% CL Hazard Ratio"	
				  HR_UpperCL="Upper &_cl.% CL Hazard Ratio"	
			 ;
		  RUN;

	 DATA msoc.&prefix0.modelinfo;
		LENGTH Description $ 40 cValue $ 50 nValue 8;

		  Description="Data Set";	
 		  cValue="&reg_ds_in.";
		   OUTPUT;

		   Description="Dependent Variable";	
 		   cValue="&dependent_vars.";
		   OUTPUT;

		    Description="Censoring Variable";	
 		   cValue="&censoring_var.";
		   OUTPUT;

			Description="Censoring Value(s)";	
 		   cValue="&censoring_lev.";
		   nValue=&censoring_lev.;
		   OUTPUT;

		    IF "&freq."  NE  " " THEN
		                   DO;
			  Description="Frequency Variable";	
 		      cValue="&freq.";
			  nValue=. ;
			  OUTPUT;
						   END;

		   IF "&weight."  NE  " " THEN
		                   DO;
			  Description="Weight Variable";	
 		      cValue="&weight.";
			  nValue=. ;
			  OUTPUT;
						   END;

			Description="Ties Handling";	
 		   cValue=UPCASE("&ties.");
		   nValue=.;
		   OUTPUT;
		RUN;
		                                           %END;	 /* End of do-group for  &iter_nb. = %eval(&iter_nb_at_conv. + 1)  */


		%IF &iter_nb. = %eval(&iter_nb_at_conv. + &numb_iter_after_conv.) %THEN
                       %DO;
			  DATA msoc.&prefix0.resid_sum_by_pct;
	                SET &prefix0.resid_sum_by_pct_c;
			  RUN;
						 %END;	 /* End of do-group for  &iter_nb. = %eval(&iter_nb_at_conv. + &numb_iter_after_conv.)  */

  %PUT;
   %PUT -----   End of the macro calc_final_stats_cox_central  iter_nb=&iter_nb. ;
   %PUT;

		%MEND;
