%MACRO calc_residual_stats_cox_dp;


  %PUT;
   %PUT -----   Start of the macro calc_residual_stats_cox_dp  iter_nb=&iter_nb. ;
   %PUT;


 /* Wrapper for calculating residuals at a DP site for Cox regression . It can only be used in the context of distributed regression macro */

%LOCAL min_count_per_grp_mod allowed_n_grp pct_incr_for_histgrm  resid_vars max_allowed_n_grp tmp_info_baseln_hazard;

/* If dp_cd is  not a part of strata variables then use &infolder_lib..&prefix0.baseln_hazardthat came from
   central site. Otherwise, use &msoc_lib..&prefix0.baseln_hazard which is calculated at this DP.
*/


	%IF &pos_dp_cd_in_strata_vars. =0 
  %THEN	%LET tmp_info_baseln_hazard= &infolder_lib..&prefix0.baseln_hazard;
%ELSE     %LET tmp_info_baseln_hazard=&msoc_lib..&prefix0.baseln_hazard;

   /* Calculate cumulative hazard contribution, survival functions and residuals on idividual
     level.  Save  the dataset in the DP's dplocal library*/

		DATA  &dplocal_lib..&prefix0_dp.resid_detail;;
		  MERGE &dplocal_lib..&prefix0_dp.reg_vars(IN=in_regr)
				         &tmp_info_baseln_hazard.(KEEP=&strata_vars. &dependent_vars. _Cum_Hazard0_	 RENAME=(_Cum_Hazard0_ = _Cum_Hazard0_b))
                 ;
			BY &strata_vars.  &dependent_vars.;
				   RETAIN _Cum_Hazard0_ 0;

				   IF &fst_by_strata_var.=1 THEN  _Cum_Hazard0_=COALESCE(_Cum_Hazard0_b,0);
				   ELSE 				_Cum_Hazard0_=COALESCE(_Cum_Hazard0_b,_Cum_Hazard0_,0);

				   IF in_regr;

			_Cum_Hazard_=EXP(_XBETA_) * _Cum_Hazard0_;
		   _SURVIVAL_=EXP(- _Cum_Hazard_);
		   Resid_Martingale=_Censoring_var_ - _Cum_Hazard_;
					  /* Ddefensive coding. */
			 IF  _Cum_Hazard_ < = 0 THEN  _tmp_for_dev=.;
		    ELSE _tmp_for_dev=ABS(-Resid_Martingale - _Censoring_var_ * LOG(_Cum_Hazard_) );
            Resid_Dev=SIGN(Resid_Martingale)*SQRT(2*_tmp_for_dev );

     _RESP_=&dependent_vars.;
	    DROP _tmp_for_dev  ;
	  RUN;

	   	PROC MEANS DATA=&msoc_lib..&prefix0.total_cnts NWAY NOPRINT;
			 VAR Total;
	       OUTPUT 	 OUT=	&prefix0.total_cnts_all( DROP=_TYPE_ _FREQ_)
							 SUM(Total)=Total
                               ;
	    RUN;
      QUIT;

	
	 DATA &prefix0_dp.allowed_N_grp;
		 SET  &prefix0.total_cnts_all(KEEP=Total RENAME=(Total=_FREQ_));
			
			    min_count_per_grp_mod=MAX(ROUND(_FREQ_/&max_numb_of_grp.), &min_count_per_grp.,1);	
			/*	 min_count_per_grp_mod=MAX(&min_count_per_grp.,1);	  */
				 max_allowed_n_grp=MAX(INT(_FREQ_/min_count_per_grp_mod),1);
				 allowed_n_grp=MIN(max_allowed_n_grp,&groups.);
				 pct_incr_for_histgrm=MAX(ROUND(100/max_allowed_n_grp,1),1);

				CALL SYMPUT("min_count_per_grp_mod",STRIP(PUT(min_count_per_grp_mod,8.)));
				CALL SYMPUT("allowed_n_grp",STRIP(PUT(allowed_n_grp,8.)));
				CALL SYMPUT("max_allowed_n_grp",STRIP(PUT(max_allowed_n_grp,8.)));
				CALL SYMPUT("pct_incr_for_histgrm",STRIP(PUT(pct_incr_for_histgrm,8.)));
		   RUN;

		   %PUT min_count_per_grp=&min_count_per_grp. ;
		   %PUT min_count_per_grp_mod=&min_count_per_grp_mod. ;
		   %PUT allowed_n_grp=&allowed_n_grp.;
		   %PUT max_allowed_n_grp=&max_allowed_n_grp.;
		   

	 /* The output dataset &prefix0_dp.resid_sum_by_pct has data groupd data by percentiles of
	   linear predictor. The number of groups is determined by &min_count_per_grp_mod.
	   and 	&allowed_n_grp. For each group (bin) the macro %sum_by_pct2 calculates
		mean of Resid_Martingale Resid_Dev and linear predictor _XBETA_;     
     */	

		   %LET resid_vars=  Resid_Martingale Resid_Dev;

		%sum_by_pct2(tbl_name_in=&dplocal_lib..&prefix0_dp.resid_detail
			                    ,tbl_name_out= &prefix0_dp.resid_sum_by_pct
								,freq=_FREQ_
			                    ,x_var= _XBETA_
                                ,y_var=&resid_vars.
								,stats_for_y_vars=%STR(MEAN= )
								,min_count_per_grp=&min_count_per_grp_mod.
                                ,groups=&allowed_n_grp.);


		DATA  &msoc_lib..&prefix0.resid_sum_by_pct;
		 LENGTH dp_cd 8;
		  SET &prefix0_dp.resid_sum_by_pct;
			 ARRAY num{*} _NUMERIC_;
		      dp_cd=COALESCE(dp_cd,&dp_cd.);
			    /* In case DP has a a very small dataset. Should almost never happen*/
			  IF &allowed_n_grp. =1 AND _FREQ_ < &min_count_per_grp_mod. THEN 
                                 DO i =1 TO DIM(num);
			 IF LOWCASE(VNAME(num{i})) NE "dp_cd" THEN num{i}=.;
                         IF _N_=1;	
								 END;
			DROP i;
		RUN;

  %PUT;
   %PUT -----   End of the macro calc_residual_stats_cox_dp  iter_nb=&iter_nb. ;
   %PUT;

%MEND;