/*HEADER------------------------------------------------------------------------
	
Calculates data for ROC curve in the context of the Logistic regression.

   Usage:

 %LET prefix0=l1_;
 %LET tbl_name_in=&prefix0.data_for_roc;
 %LET tbl_name_out=msoc.&prefix0.ROC;

                 %calculate_roc( tbl_name_in=&prefix0.data_for_roc
                                              ,tbl_name_out=msoc.&prefix0.ROC
                                              ,prefix0=dl1_);
 	First version:  07/09/2016, Yury Vilk. 
+------------------------------------------------------------------------HEADER*/

  %MACRO calc_roc( tbl_name_in=,tbl_name_out=);
 
  PROC DATASETS NOLIST NOWARN;
	DELETE resid_for_roc1 cum_resp_detail1 cum_resp_sum1;
  RUN;

 DATA resid_for_roc1;
	  LENGTH  _PROB_ 8;
			   SET &tbl_name_in.(KEEP= _PROB_  _FREQ_  _RESP_) END=last;

					  OUTPUT;
	/* Add _PROB_ 0 and 1 with zero _FREQ_ and _RESP_. 
	  Logistic cannot have such probilities but they are needed for ROC curve
	*/
			   IF last  THEN 
		                   DO;
				 _PROB_=1;
				 _FREQ_= 0;
				 _RESP_= 0;
				   OUTPUT;
				 _PROB_=0;
				  OUTPUT;	
				     END;
						;
			 RUN;
 
PROC SORT DATA=resid_for_roc1;
	BY DESCENDING _PROB_  ;
	WHERE _PROB_ IS NOT NULL;
  RUN; ;

  DATA cum_resp_detail1(KEEP= _PROB_ CUM_RESP_  Cum_FREQ_  _FREQ_)
			cum_resp_sum1(KEEP=  _Total_RESP_   _Total_FREQ_ )
      ;
	 LENGTH _PROB_ Cum_FREQ_  CUM_RESP_     8;

      SET resid_for_roc1 END=last;
	    BY DESCENDING _PROB_  ;
  
     RETAIN CUM_RESP_ 0  Cum_FREQ_	 0;

    CUM_RESP_ = SUM(CUM_RESP_  ,  _RESP_);
	Cum_FREQ_=SUM(Cum_FREQ_ , _FREQ_ );

    IF last THEN 
               DO; 
     _Total_RESP_=CUM_RESP_;
	 _Total_FREQ_ =Cum_FREQ_;
	 OUTPUT cum_resp_sum1;
	          END;
	  IF LAST._PROB_ THEN  OUTPUT cum_resp_detail1;	
	RUN;
	 
	 DATA &tbl_name_out.;
	  LENGTH  _PROB_	 _POS_  _NEG_  _FALPOS_  _FALNEG_ Cum_FREQ_ _SENSIT_ _1MSPEC_  _AUC_ 8;
	  SET cum_resp_detail1(RENAME=(Cum_RESP_=_POS_));
		  RETAIN _Total_POS_ _Total_FALPOS_  _AUC_ 0;

	  		_FALPOS_ = SUM(Cum_FREQ_ , - _POS_ );

	  IF _N_=1 THEN
                   DO;
			SET cum_resp_sum1;
			  _Total_POS_=_Total_RESP_;
			  _Total_FALPOS_=_Total_FREQ_ -_Total_RESP_;
				   END;	

	   _NEG_=_Total_FALPOS_- _FALPOS_;
	   _FALNEG_=_Total_POS_ - _POS_;

		_SENSIT_=_POS_/_Total_POS_;
		_1MSPEC_=_FALPOS_/_Total_FALPOS_;

		  LAG_SENSIT_ =LAG(	_SENSIT_) ;
		 LAG_1MSPEC_ =LAG(	_1MSPEC_) ;

		 /* Calculate area under the curve using trapezoidal rule. 
		   The  last observation (_PROB_=0) will have the answer. 
		 */
		  IF _N_ > 1 THEN
                   DO;
			 _AUC_=_AUC_ + 0.5*(_SENSIT_+LAG_SENSIT_)*(_1MSPEC_ - LAG_1MSPEC_);
				   END;	
	 KEEP _PROB_	 _POS_  _NEG_  _FALPOS_  _FALNEG_ Cum_FREQ_ _SENSIT_ _1MSPEC_  _AUC_	  ;

	 LABEL _SENSIT_="Sensitivity" 	_1MSPEC_="1 - Specificity";
	RUN;

	%MEND;