
/*		The input table must have a columns full_file_name and file_name  (short name).
   The macro adds three columns to the output table: 

    file_exist_flag 
    file_mod_dtm   (datetime when a file was modified)
    utc_offset_display (offset for the time zone of the computer relative to the universal time);

Usage:

%LET prefix_dp=dr1_i1_dp1_;
%find_files_mod_dtm( tbl_name_in=&prefix_dp.file_list_out , tbl_name_out=&prefix_dp.file_list_out_dtm_t);

 %find_files_mod_dtm( tbl_name_in=&prefix2.file_list , tbl_name_out=&msoc_lib..&prefix2.file_list);

 First version:  02/08/2016, Yury Vilk. 
 */

%MACRO find_files_mod_dtm( tbl_name_in=, tbl_name_out=);
	 %GLOBAL missing_or_0_obs_files missing_files_sh missing_files;

   %PUT;
   %PUT -----   Start of the macro find_files_mod_dtm;
   %PUT;


DATA &tbl_name_out.;
		  SET &tbl_name_in.;

		  	 FORMAT file_exist_flag 1.  file_mod_dtm DATETIME21.2 utc_offset_display $6.;
			 LENGTH   tmp_fref $ 8 MESSAGE $ 5000;

	file_exist_flag= FILEEXIST(full_file_name);
					 
			    IF file_exist_flag = 1 THEN
										DO;
			 tmp_fref="t_fref" ;
			 rc_f=FILENAME(tmp_fref,full_file_name);
			 _fid=fopen(tmp_fref);

		 IF _fid=0 THEN DO; MESSAGE=sysmsg();  PUT _fid= tmp_fref= MESSAGE=; END;

	/* Property "Last Modified" currently supported only on Windows. 
	     For other OS file_mod_dtm will be missing 
	*/
			 file_mod_dtm=INPUT(STRIP(finfo(_fid, "Last Modified")), ?? ANYDTDTM32.);
	/*		 file_mod_dtm=INPUT(STRIP(finfo(_fid, "Last Modified")),DATETIME21.); */
			 rc_close=fclose(_fid) ;
			 rc_f_clear=FILENAME(tmp_fref);	

			 IF ext="sas7bdat" THEN 
                 DO;
			 dsid=open(STRIP(full_file_name), , , "F");
			 IF dsid=0 THEN DO; MESSAGE=sysmsg();  PUT dsid=  MESSAGE=; END;
             nobs=attrn(dsid,"NLOBS") ;
			 rc_ds_close=CLOSE(dsid);
			    END;
			 						  END;

			utc_offset_display=substr(put('00:00:00't,E8601LZ.), 9);

	*	DROP tmp_fref  rc_f rc_close rc_f_clear  _fid dsid rc_ds_close MESSAGE;

RUN;
 
  %LET missing_or_0_obs_files=;
   %LET missing_files=;
  %LET missing_files_sh=;

  PROC SQL NOPRINT;
	  SELECT file_nm AS missing_or_0_obs_files
					,CASE
					  	WHEN file_exist_flag=0 THEN file_nm
						ELSE " "
					END	   AS  missing_files_sh
				   ,CASE
					  	WHEN file_exist_flag=0 THEN full_file_name
						ELSE  " " 
					END	  AS missing_files

            	INTO :missing_or_0_obs_files	SEPARATED BY " "
                        ,:missing_files_sh	          SEPARATED BY " "
                        , :missing_files				 SEPARATED BY " "
			 
	     FROM  &tbl_name_out.
		  WHERE( file_exist_flag=1 AND nobs=0)
		   OR  file_exist_flag=0
			 ;
	 QUIT;
 %IF &sqlobs. >0 %THEN	%PUT Number of files that do not exist or SAS datasets with zero observation: &sqlobs. ;

  %PUT;
   %PUT -----   End of the macro find_files_mod_dtm;
   %PUT;

%MEND; /* End of find_files_mod_dtm  */






